Module: Dynflow::Action::Format
- Included in:
- Dynflow::Action
- Defined in:
- lib/dynflow/action/format.rb
Instance Method Summary collapse
Instance Method Details
#input_format(&block) ⇒ Object
5 6 7 8 |
# File 'lib/dynflow/action/format.rb', line 5 def input_format(&block) # Format definitions are not validated # This method is kept for backward compatibility but does nothing end |
#output_format(&block) ⇒ Object
10 11 12 13 |
# File 'lib/dynflow/action/format.rb', line 10 def output_format(&block) # Format definitions are not validated # This method is kept for backward compatibility but does nothing end |